/* NIGHTFALL Light Curve Synthesis Program                                 */
/* Copyright (C) 1998 Rainer Wichmann                                      */
/*                                                                         */
/*  This program is free software; you can redistribute it                 */
/*  and/or modify                                                          */
/*  it under the terms of the GNU General Public License as                */
/*  published by                                                           */
/*  the Free Software Foundation; either version 2 of the License, or      */
/*  (at your option) any later version.                                    */
/*                                                                         */
/*  This program is distributed in the hope that it will be useful,        */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/*  GNU General Public License for more details.                           */
/*                                                                         */
/*  You should have received a copy of the GNU General Public License      */
/*  along with this program; if not, write to the Free Software            */
/*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.              */

/* --------------------------------------------------------------- */

#include <math.h>
#include <stdio.h>
#include <string.h>
#include <float.h>

#include "Light.h"
#include "LightPassband.h"

/****************************************************************************
 @package   nightfall
 @author    Rainer Wichmann (rwichman@lsw.uni-heidelberg.de)
 @version   1.0
 @short     Blackbody flux for an element
 @param     (SurfaceElement  *) SurfPtr  the surface element
 @return    (void)        
 @heading   Flux Computation
 ****************************************************************************/
static double BBcalc (double lambda, double t)
{
  static  double  CC1 = 3.7418498e+15;/* 10e(5*4)      erg cm / sec       */
  static  double  CC2 = 14388.3361;   /* 10e4          cm K               */
                                      /* WaveL in 10e4 cm                 */
  /* we compute I(1) -- Intensity normal to surface                   */
  /*  in blackbody approximation                                      */  
  /* B = CC1/WaveL^5 * 1/(exp(CC2/(T*WaveL))-1) WaveL(ength) in _cm_  */

  return ( (CC1/(lambda * lambda * lambda * lambda * lambda))
	   * 1.0 / (exp( CC2/(lambda * t))-1.0) );
}
     
static void BBFlux(SurfaceElement  *SurfPtr, int Comp)
{
  OrbitType *OrbPtr = &Orbit;         /* orbit pointer                    */
  int     band;                       /* the passband                     */
  double  Temp;                       /* temperature                      */
  double  ScaledArea;                 /* scaled surface area              */

  /* -----------  surface rescaling --------------------------------- */
  
  ScaledArea = SurfPtr->area * SQR(OrbPtr->Dist); 
  
  /* -----------  flux computation  --------------------------------- */
  
  Temp = SurfPtr->temp;

  for (band = 0; band < NUM_MAG; ++band) 
    {
      SurfPtr->f_[band] = ScaledArea * BBcalc(WaveL[Comp][band], Temp);
    }
  return;
}

/****************************************************************************
 @package   nightfall
 @author    Rainer Wichmann (rwichman@lsw.uni-heidelberg.de)
 @version   1.0
 @short     Computation of blackbody fluxes
 @param     (int)  Comp        The stellar component
 @return    (void)        
 @heading   Flux Computation
 ****************************************************************************/
void BlackbodyFlux(int Comp)
{
  SurfaceElement  *SurfPtr;           /* pointer to surface               */
  long    i;                          /* loop variable                    */

  SurfPtr = Surface[Comp];

  for (i = 0; i < Binary[Comp].NumElem; ++i) 
    { 
      BBFlux(SurfPtr, Comp);
      ++SurfPtr;
    }
}


/****************************************************************************
 @package   nightfall
 @author    Rainer Wichmann (rwichman@lsw.uni-heidelberg.de)
 @version   1.0
 @short     Computation of atmospheric model fluxes
 @param     (int)  Comp        The stellar component
 @return    (void)        
 @heading   Flux Computation
 ****************************************************************************/
void ModelFlux(int Comp)
{
  SurfaceElement * SurfPtr;           /* pointer to surface               */
  OrbitType *OrbPtr = &Orbit;         /* orbit pointer                    */
  
  register int     j;                 /* loop variable                    */
  register long    i;                 /* loop variable                    */

  double           T;                 /* temperature                      */
  double           ScaledArea;        /* scaled surface area              */
  double           ScaleFactor;       /* scale factor for surface area    */
  unsigned int     n_size;
  int              n_where;
  float            n_flux;
  float          * n_tarr;
  float          * n_farr;
  unsigned int     bb_fallback[NUM_MAG] = { 0 };
  
  /* -----------   initialize data   ---------------------------------    */

  SurfPtr = Surface[Comp];

  /* 
   * scale factor for surface area
   */
  ScaleFactor = SQR(OrbPtr->Dist);

  /* -----------   main loop over surface   --------------------------    */

  for (i = 0; i < Binary[Comp].NumElem; ++i) {
    
    T          = SurfPtr->temp;
    ScaledArea = SurfPtr->area * ScaleFactor;

    for (j = 0; j < NUM_MAG; ++j)
      {
	n_size   = nf_modelFlux(j, Binary[Comp].log_g, T,
				&n_tarr, &n_farr);

	if (n_size != 0)
	  {
	    n_flux = nf_interpolate(T, n_tarr, n_farr, n_size, &n_where);
	    if (isnan(n_flux) || n_flux < 0.0)
	      {
		n_flux = 0.0;
		fprintf(stderr, "internal error in nf_interpolate\n");
	      }
	  }
	else
	  {
	    n_flux = BBcalc(WaveL[Comp][j], T) *
	      nf_bb_adjust(j, Binary[Comp].log_g, T);
	    ++bb_fallback[j];
	  }
	
	SurfPtr->f_[j] = (ScaledArea * n_flux);
      }
    
    ++SurfPtr;
  }

#if !defined(NDEBUG)
  for (j = 0; j < NUM_MAG; ++j)
    if (bb_fallback[j] > 0)
      {
	fprintf(stderr,
		"blackbody fallback used for %s (passband %2d, %u/%ld surface elements)\n",
		Component[Comp], j, bb_fallback[j], Binary[Comp].NumElem);
      }
#endif
  
  return;
}




