/* NIGHTFALL Light Curve Synthesis Program                                 */
/* Copyright (C) 1998 Rainer Wichmann                                      */
/*                                                                         */
/*  This program is free software; you can redistribute it                 */
/*  and/or modify                                                          */
/*  it under the terms of the GNU General Public License as                */
/*  published by                                                           */
/*  the Free Software Foundation; either version 2 of the License, or      */
/*  (at your option) any later version.                                    */
/*                                                                         */
/*  This program is distributed in the hope that it will be useful,        */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/*  GNU General Public License for more details.                           */
/*                                                                         */
/*  You should have received a copy of the GNU General Public License      */
/*  along with this program; if not, write to the Free Software            */
/*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.              */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <float.h>
#include <assert.h>

#include "Light.h"
#include "LightPassband.h"

static double DBoost[3][NUM_MAG];

/****************************************************************************
 @package   nightfall
 @author    Rainer Wichmann (rwichmann@hs.uni-hamburg.de)
 @version   1.0
 @short     Compute limb darkening coefficients 
 @long      Limb darkening coefficients 
            from Claret A. 2000 Astron. Astrophys. 363, 1081
 @tip       log(g) = 3.5, 4.0, 4.5, 5.0
 @param     (int) Comp The stellar component
 @return    (void) 
 @heading   Light Curve
 ****************************************************************************/

static void FindbestLC ();

static void LightLimbDark_int(int Comp);
/* component, passband, number */
static double ExtLimb[NUM_COMP][NUM_MAG][NF_PB_LMAX];
static int externalLimb[NUM_COMP] = { 0 };

/* This function may be called from the command line
 * using the '-l[P/S] lc_1_a,lc_1_b,..,lc_12_b' option
 */
int  SetLimbCoeff(int Comp, const char * str)
{
  int    j, band;
  double dval;
  char * p;

  size_t len = strlen(str);
  char * xs = malloc(len+1);

  strcpy(xs, str);

  for (band = 0; band < NUM_MAG; ++band) {

    for (j = 0; j < 2; ++j) {
      p = ((band == 0) && (j == 0)) ? strtok(xs, ",") : strtok(NULL, ",");
      if (p) {
	dval = atof(p);
	ExtLimb[Comp][band][j] = dval;
      }
      else {
	return -1;
      }
    }
  }

  free(xs);
  externalLimb[Comp] = 1;

  return 0;
}

void LightLimbDark(int Comp)
{
  register long    j;                        /* loop variable               */ 
  int      band;                             /* passband                    */

#ifdef HAVE_DISK
  if (Comp == Disk)
    Binary[Comp].log_g = 3.5;
#endif
  
  if (externalLimb[Comp])
    {
      for (band = 0; band < NUM_MAG; ++band) {
	for (j = 0; j < 2; ++j) {
	  Limb[Comp][band][j] = ExtLimb[Comp][band][j];
	}
      }
    }
  else
    {
      LightLimbDark_int(Comp);
    }

  /* -----------   Verbose output ------------------------------------- */

  if (Flags.debug[VERBOSE] == ON) {

    printf(_("Limb Darkening Coefficients: \n"));

    if(Comp == Primary) {
      printf(_(" Primary: \n"));
      printf("  %16s: %6.3f %6.3f %16s: %6.3f %6.3f %16s: %6.3f %6.3f\n",
	     nf_pb_get_name(Umag), Limb[Primary][Umag][0],Limb[Primary][Umag][1],
	     nf_pb_get_name(Bmag), Limb[Primary][Bmag][0],Limb[Primary][Bmag][1],
	     nf_pb_get_name(Vmag), Limb[Primary][Vmag][0],Limb[Primary][Vmag][1]);
      printf("  %16s: %6.3f %6.3f %16s: %6.3f %6.3f \n",
	     nf_pb_get_name(umag), Limb[Primary][umag][0],Limb[Primary][umag][1],
	     nf_pb_get_name(vmag), Limb[Primary][vmag][0],Limb[Primary][vmag][1]);
      printf("  %16s: %6.3f %6.3f %16s: %6.3f %6.3f\n",
	     nf_pb_get_name(bmag), Limb[Primary][bmag][0],Limb[Primary][bmag][1],
	     nf_pb_get_name(ymag), Limb[Primary][ymag][0],Limb[Primary][ymag][1]);
      printf("  %16s: %6.3f %6.3f %16s: %6.3f %6.3f\n",
	     nf_pb_get_name(Rmag), Limb[Primary][Rmag][0],Limb[Primary][Rmag][1],
	     nf_pb_get_name(Imag), Limb[Primary][Imag][0],Limb[Primary][Imag][1]);
      printf("  %16s: %6.3f %6.3f %16s: %6.3f %6.3f %16s: %6.3f %6.3f\n",
	     nf_pb_get_name(Jmag), Limb[Primary][Jmag][0],Limb[Primary][Jmag][1],
	     nf_pb_get_name(Hmag), Limb[Primary][Hmag][0],Limb[Primary][Hmag][1],
	     nf_pb_get_name(Kmag), Limb[Primary][Kmag][0],Limb[Primary][Kmag][1]);
    }
    
    if(Comp == Secondary) {   
      printf(_(" Secondary: \n"));
      printf("  %16s: %6.3f %6.3f %16s: %6.3f %6.3f %16s: %6.3f %6.3f\n",
	     nf_pb_get_name(Umag), Limb[Secondary][Umag][0],Limb[Secondary][Umag][1],
	     nf_pb_get_name(Bmag), Limb[Secondary][Bmag][0],Limb[Secondary][Bmag][1],
	     nf_pb_get_name(Vmag), Limb[Secondary][Vmag][0],Limb[Secondary][Vmag][1]);
      printf("  %16s: %6.3f %6.3f %16s: %6.3f %6.3f \n",
	     nf_pb_get_name(umag), Limb[Secondary][umag][0],Limb[Secondary][umag][1],
	     nf_pb_get_name(vmag), Limb[Secondary][vmag][0],Limb[Secondary][vmag][1]);
      printf("  %16s: %6.3f %6.3f %16s: %6.3f %6.3f\n",
	     nf_pb_get_name(bmag), Limb[Secondary][bmag][0],Limb[Secondary][bmag][1],
	     nf_pb_get_name(ymag), Limb[Secondary][ymag][0],Limb[Secondary][ymag][1]);
      printf("  %16s: %6.3f %6.3f %16s: %6.3f %6.3f\n",
	     nf_pb_get_name(Rmag), Limb[Secondary][Rmag][0],Limb[Secondary][Rmag][1],
	     nf_pb_get_name(Imag), Limb[Secondary][Imag][0],Limb[Secondary][Imag][1]);
      printf("  %16s: %6.3f %6.3f %16s: %6.3f %6.3f %16s: %6.3f %6.3f\n",
	     nf_pb_get_name(Jmag), Limb[Secondary][Jmag][0],Limb[Secondary][Jmag][1],
	     nf_pb_get_name(Hmag), Limb[Secondary][Hmag][0],Limb[Secondary][Hmag][1],
	     nf_pb_get_name(Kmag), Limb[Secondary][Kmag][0],Limb[Secondary][Kmag][1]);
    }
    
#ifdef HAVE_DISK
    if(Comp == Disk) {   
      printf(_(" Disk: \n"));
      printf("  %16s: %6.3f %6.3f %16s: %6.3f %6.3f %16s: %6.3f %6.3f\n",
	     nf_pb_get_name(Umag), Limb[Disk][Umag][0],Limb[Disk][Umag][1],
	     nf_pb_get_name(Bmag), Limb[Disk][Bmag][0],Limb[Disk][Bmag][1],
	     nf_pb_get_name(Vmag), Limb[Disk][Vmag][0],Limb[Disk][Vmag][1]);
      printf("  %16s: %6.3f %6.3f %16s: %6.3f %6.3f \n",
	     nf_pb_get_name(umag), Limb[Disk][umag][0],Limb[Disk][umag][1],
	     nf_pb_get_name(vmag), Limb[Disk][vmag][0],Limb[Disk][vmag][1]);
      printf("  %16s: %6.3f %6.3f %16s: %6.3f %6.3f\n",
	     nf_pb_get_name(bmag), Limb[Disk][bmag][0],Limb[Disk][bmag][1],
	     nf_pb_get_name(ymag), Limb[Disk][ymag][0],Limb[Disk][ymag][1]);
      printf("  %16s: %6.3f %6.3f %16s: %6.3f %6.3f\n",
	     nf_pb_get_name(Rmag), Limb[Disk][Rmag][0],Limb[Disk][Rmag][1],
	     nf_pb_get_name(Imag), Limb[Disk][Imag][0],Limb[Disk][Imag][1]);
      printf("  %16s: %6.3f %6.3f %16s: %6.3f %6.3f %16s: %6.3f %6.3f\n",
	     nf_pb_get_name(Jmag), Limb[Disk][Jmag][0],Limb[Disk][Jmag][1],
	     nf_pb_get_name(Hmag), Limb[Disk][Hmag][0],Limb[Disk][Hmag][1],
	     nf_pb_get_name(Kmag), Limb[Disk][Kmag][0],Limb[Disk][Kmag][1]);
    }
#endif
  }

  return;
}

static void checkLDC()
{
  static int      fail = 0;
  int      failold;
  int      band;

  failold = fail;
  if ((Flags.limb == 0) && (0 == (fail & LD_LINEAR))) 
    for (band = 0; band < NUM_MAG; ++band) {
      if (0 ==  nf_pb_has_ldType(band, LD_LINEAR)) fail |= LD_LINEAR; }
  else if ((Flags.limb == 1) && (0 == (fail & LD_SQR))) 
    for (band = 0; band < NUM_MAG; ++band) {
      if (0 ==  nf_pb_has_ldType(band, LD_SQR)) fail |= LD_SQR; }
  else if ((Flags.limb == 2) && (0 == (fail & LD_SQRT)))
    for (band = 0; band < NUM_MAG; ++band) {
      if (0 ==  nf_pb_has_ldType(band, LD_SQRT)) fail |= LD_SQRT; }
  else if ((Flags.limb == 3) && (0 == (fail & LD_FOUR)))
    for (band = 0; band < NUM_MAG; ++band) {
      if (0 ==  nf_pb_has_ldType(band, LD_FOUR)) fail |= LD_FOUR; }

  if (fail != failold)
    {
      char * msg = _("Limb darkening model not supported for some passbands");
      WARNING(msg);
    }
  return;
}

static void LightLimbDark_int(int Comp)
{
  float    ArrLDC[NF_PB_LMAX];
  float  * result;
  float    Tn;                               /* temperature             */
  register long    j;                        /* loop variable           */ 
  int      band;                             /* passband                */
  double   log_g = Binary[Comp].log_g;       /* log g                   */

  
  /* -----------  initialize ------------------------------------------ */

  checkLDC(); /* is selected ldc model supported?                       */

  /* Default value is zero (no limb darkening)                          */
  
  for (band = 0; band < NUM_MAG; ++band) 
    for (j = 0; j < NF_PB_LMAX; ++j) 
      Limb[Comp][band][j] = 0.0;

  /* we approximate T = Binary[Comp].Temperature for all                */
  /* surface elements                                                   */

  Tn  = (float) Binary[Comp].Temperature;
  
  /* -----------   linear law ----------------------------------------- */

  if (Flags.limb == 0)
    {
      for (band = 0; band < NUM_MAG; ++band)
	{
	  result = nf_pb_fill_ldArray(band, LD_LINEAR, (float) log_g, Tn, ArrLDC);

	  if (result)
	    Limb[Comp][band][0] = result[0];
	}
    }

  /* -----------   quadratic law--------------------------------------- */

  else if (Flags.limb == 1)
    {
      for (band = 0; band < NUM_MAG; ++band)
	{
	  result = nf_pb_fill_ldArray(band, LD_SQR, (float) log_g, Tn, ArrLDC);

	  if (result) {
	    Limb[Comp][band][0] = result[0];
	    Limb[Comp][band][1] = result[1]; }
	}
    }

  /* -----------   square root law------------------------------------- */

  else if (Flags.limb == 2)
    {
      for (band = 0; band < NUM_MAG; ++band)
	{
	  result = nf_pb_fill_ldArray(band, LD_SQRT, (float) log_g, Tn, ArrLDC);

	  if (result) {
	    Limb[Comp][band][0] = result[0];
	    Limb[Comp][band][1] = result[1]; }
	}
    }
 
  /* -----------   four param     ------------------------------------- */

  else if (Flags.limb == 3)
    {
      for (band = 0; band < NUM_MAG; ++band)
	{
	  result = nf_pb_fill_ldArray(band, LD_FOUR, (float) log_g, Tn, ArrLDC);

	  if (result) {
	    Limb[Comp][band][0] = result[0];
	    Limb[Comp][band][1] = result[1];
	    Limb[Comp][band][2] = result[2];
	    Limb[Comp][band][3] = result[3];
	  }
	}
    }
 
  /* -----------   Modify, if using monochrome wavelengths ------------ */

  if (Flags.monochrome == ON)
    FindbestLC ();

  return;
}



/****************************************************************************
 @package   nightfall
 @author    Rainer Wichmann (rwichman@lsw.uni-heidelberg.de)
 @version   1.0
 @short     Verbose output of wavelengths
 @param     (int) Comp The stellar component
 @return    (void) 
 @heading   Light Curve
 ****************************************************************************/
static
void printData(int Comp)
{
  printf("  %16s: %6.4f %16s: %6.4f %16s: %6.4f %16s: %6.4f %16s: %6.4f\n",
	 nf_pb_get_name(Umag), WaveL[Comp][Umag], nf_pb_get_name(Bmag), WaveL[Comp][Bmag],
	 nf_pb_get_name(Vmag), WaveL[Comp][Vmag], nf_pb_get_name(Rmag), WaveL[Comp][Rmag],
	 nf_pb_get_name(Imag), WaveL[Comp][Imag]);
  printf("  %16s: %6.4f %16s: %6.4f %16s: %6.4f %16s: %6.4f\n",
	 nf_pb_get_name(umag), WaveL[Comp][umag], nf_pb_get_name(vmag), WaveL[Comp][vmag],
	 nf_pb_get_name(bmag), WaveL[Comp][bmag], nf_pb_get_name(ymag), WaveL[Comp][ymag]);
  printf("  %16s: %6.4f %16s: %6.4f %16s: %6.4f\n\n",
	 nf_pb_get_name(Jmag), WaveL[Comp][Jmag], nf_pb_get_name(Hmag), WaveL[Comp][Hmag],
	 nf_pb_get_name(Kmag), WaveL[Comp][Kmag]);
  
  printf("  %16s: %6.4f %16s: %6.4f %16s: %6.4f %16s: %6.4f %16s: %6.4f\n",
	 nf_pb_get_name(Umag), DBoost[Comp][Umag], nf_pb_get_name(Bmag), DBoost[Comp][Bmag],
	 nf_pb_get_name(Vmag), DBoost[Comp][Vmag], nf_pb_get_name(Rmag), DBoost[Comp][Rmag],
	 nf_pb_get_name(Imag), DBoost[Comp][Imag]);
  printf("  %16s: %6.4f %16s: %6.4f %16s: %6.4f %16s: %6.4f\n",
	 nf_pb_get_name(umag), DBoost[Comp][umag], nf_pb_get_name(vmag), DBoost[Comp][vmag],
	 nf_pb_get_name(bmag), DBoost[Comp][bmag], nf_pb_get_name(ymag), DBoost[Comp][ymag]);
  printf("  %16s: %6.4f %16s: %6.4f %16s: %6.4f\n",
	 nf_pb_get_name(Jmag), DBoost[Comp][Jmag], nf_pb_get_name(Hmag), DBoost[Comp][Hmag],
	 nf_pb_get_name(Kmag), DBoost[Comp][Kmag]);
  return;
}

static
void VerboseMessage (int Comp)
{
  if (Flags.debug[VERBOSE] == ON) {
    printf("\n");
    printf(_("Effective Wavelengths / Doppler Boost Factor:\n"));
    if(Comp == Primary) { 
      printf(_(" Primary: \n"));
    }
    else if(Comp == Secondary) {
      printf(_(" Secondary: \n"));
    }
#ifdef HAVE_DISK
    else if(Comp == Disk) {
      printf(_(" Disk: \n"));
    }
#endif
    printData(Comp);
  }

  return;
}

/****************************************************************************
 @package   nightfall
 @author    Rainer Wichmann (rwichman@lsw.uni-heidelberg.de)
 @version   1.0
 @short     Set monochromatic wavelengths for blackbody
 @param     (int) Comp The stellar component
 @return    (void) 
 @heading   Light Curve
 ****************************************************************************/

static
void SetMonoWavelengths(int Comp)
{
  int band;

  for (band = 0; band < NUM_MAG; ++band) 
    {
      WaveL[Comp][band] = nf_pb_get_wave(band);
    }

  /* Verbose output                                                      */

  VerboseMessage(Comp);
  
  return;
}

/****************************************************************************
 @package   nightfall
 @author    Rainer Wichmann (rwichman@lsw.uni-heidelberg.de)
 @version   1.0
 @short     Define monochromatic wavelengths for blackbody
 @param     (void)
 @return    (int) Flag ON/OFF
 @heading   Light Curve
 ****************************************************************************/
float mono_zero[NUM_MAG];

int DefineMonoWavelengths ()
{
  int    band;
  int    len;
  float  fin;
  char * pp;
  char * qq;

  for (band = 0; band < NUM_MAG; ++band)
    mono_zero[band] = nf_pb_get_wave(band);
  
  if (NULL == (pp = getenv("NIGHTFALL_MONO_WAVE")))
    return OFF;

  len = strlen(pp) + 1;

  qq = (char *) malloc (len);

  if (NULL == qq)
    {
      nf_error(_(errmsg[0]));
      return OFF;
    }

  strncpy (qq, pp, len);
  qq[len-1] = '\0';

  band = 0;
  pp = strtok(qq, ",");

  if (pp != NULL)
    {
      fin = atof(pp);
      if (fin > 0) nf_pb_set_wave(band, fin);
      ++band;
    }
  else
    return OFF;

  while (pp != NULL && band < NUM_MAG)
    {
      pp = strtok(NULL, ",");
      if (pp != NULL)
	{
	  fin = atof(pp);
	  if (fin > 0) nf_pb_set_wave(band, fin);
	  ++band;
	}
    }
     
  return ON;
}



/****************************************************************************
 @package   nightfall
 @author    Rainer Wichmann (rwichman@lsw.uni-heidelberg.de)
 @version   1.0
 @short     Search closest value in an array
 @param     (float)   fin    A float value
 @param     (float *) farr   A float array
 @param     (int)     arrsiz Size of farr
 @return    (int)     The index of closest match
 @heading   Light Curve
 ****************************************************************************/
static
int BestIndex (float fin, float * farr, int arrsiz)
{
  register int k;
  int          i = 0;
  float        tmp;
  float        best = fabs (fin - farr[0]);
  
  for (k = 1; k < arrsiz; ++k)
    {
      tmp = fabs (fin - farr[k]);
      if (tmp < best)
	{
	  best = tmp;
	  i    = k;
	}
    }
  return i;
}
    
/****************************************************************************
 @package   nightfall
 @author    Rainer Wichmann (rwichman@lsw.uni-heidelberg.de)
 @version   1.0
 @short     Find best limb darkening coefficient for monochrome wavelengths
 @param     (void)  
 @return    (void)
 @heading   Light Curve
 ****************************************************************************/
static void FindbestLC ()
{
  register int i, j, k, l;
  float  l_zero[NUM_MAG];
  double tmp_limb[NUM_COMP][NUM_MAG][NF_PB_LMAX];
  
  /* Initialize the l_zero array */
  for (k = 0; k < NUM_MAG; ++k)
    l_zero[k] = nf_pb_get_wave(k);
  
  for (k = 0; k < NUM_MAG; ++k)
    {
      /* Searches the best match for mono_zero[k] in l_zero */
      i = BestIndex (mono_zero[k], l_zero, NUM_MAG);

      for (j = 0; j < NUM_COMP; ++j)
	{
	  for (l = 0; l < NF_PB_LMAX; ++l)
	    tmp_limb[j][k][l] = Limb[j][i][l];
	}
    }
    

  /* copy from temporary array
   */
  for (i = 0; i < NUM_COMP; ++i)
    for (j = 0; j < NF_PB_LMAX; ++j)
      for (k = 0; k < NUM_MAG; ++k)
	Limb[i][k][j] = tmp_limb[i][k][j];

  return;
}
  
/****************************************************************************
 @package   nightfall
 @author    Rainer Wichmann (rwichman@lsw.uni-heidelberg.de)
 @version   1.0
 @short     Compute temperature-dependent doppler boosting factor
 @long      Uses Eq. (3) from: van Kerkwijk, M. H. et al. 2010, 
            Astrophysical Journal 715, 51
 @param     (int) Comp The stellar component
 @return    (void) 
 @heading   Light Curve
 ****************************************************************************/
static void DopplerBoostFactor(int Comp)
{
  int    band;             /* passband                                      */

  double hc = 299792458.0 * 6.626070e-34; /* h*c */
  double  k = 1.380649e-23;

  for (band = 0; band < NUM_MAG; ++band) {
    double wavel = WaveL[Comp][band] * 1.0e-6; /* eff. wavelength in meter  */
    double teff  = Binary[Comp].Temperature;   /* Kelvin                    */

    double factor = hc/(k*wavel*teff);
    DBoost[Comp][band] = factor * exp(factor) / (exp(factor) - 1.0);
  }
  return;
}

/****************************************************************************
 @package   nightfall
 @author    Rainer Wichmann (rwichman@lsw.uni-heidelberg.de)
 @version   1.0
 @short     Compute temperature-dependent effective wavelengths for blackbody
 @long      Uses second moments of passbands following the prescription by:
            Young A.T. 1992, Astronomy and Astrophysics 257, 366 (Equation 12)
            Effective wavelengths 
            calculated following  Equation 3.30 in: 
            Budding E. 1993, An Introduction to Astronomic Photometry, 
            Cambridge University Press 
            Result is in micrometers
 @param     (int) Comp The stellar component
 @return    (void) 
 @heading   Light Curve
 ****************************************************************************/
void EffectiveWavelength(int Comp)
{
  double l_max;            /* blackbody peak                             */
  int    band;             /* passband                                   */
  double Wien = 2897.756;  /* constant for Wien's law                    */

  /* second moments of passbands                                         */
  /* approximated following the prescription by                          */
  /* Young A.T. 1992, Astronomy and Astrophysics 257, 366; Equation 12   */

  float  M2ND[NUM_MAG];
  float  l_zero[NUM_MAG];
  int    k;
  
  /* Initialize the M2ND, l_zero arrays                                  */
  for (k = 0; k < NUM_MAG; ++k)
    {
      l_zero[k] = nf_pb_get_wave(k);
      M2ND[k]   = nf_pb_get_m2nd(k);
    }

  /* --------------------- monochromatic wavelength -------------------- */

  if (Flags.monochrome == ON)
    {
      /* sets the effective wavelengths WaveL[][] to user-defined values */
      SetMonoWavelengths(Comp);
      DopplerBoostFactor(Comp);
      return;
    }

  /* --------------------- effective wavelength ------------------------ */
  /* result is in micrometers                                            */

  /* effective wavelengths                                               */
  /* calculated following  Equation 3.30 in                              */
  /* Budding E. 1993, An Introduction to Astronomic Photometry           */
  /*      Cambridge University Press                                     */

  /* blackbody peak                                                      */

  l_max = Wien/Binary[Comp].Temperature;

  for (band = 0; band < NUM_MAG; ++band) {
    WaveL[Comp][band] = l_zero[band] 
      + 5*(l_max-l_zero[band])*M2ND[band]/SQR(l_zero[band]);   
  }

  /* Use effective wavelength for doppler boost factor                   */

  DopplerBoostFactor(Comp);
 
  /* Verbose output                                                      */

  VerboseMessage(Comp);

  return;
}

void DopplerBoostFlux()
{
  unsigned long j;
  unsigned int  band;
  unsigned int  Comp;

  int           sign;
    
  /* L90[component][band] is luminosity at norm. point per component        */

  if (Flags.elliptic == ON) {
    sign = -1; /* Correction is -f(DB) * (v/c)                  */
  } else {
    sign =  1;
  }
  
  /* ---------------- Doppler Boost Correction ----------------------------- */

  for (Comp = 0; Comp < 2; ++Comp)
    {
      for (j = 0; j < N_FluxOutFull; ++j) 
	{   
	  for (band = 0; band < NUM_MAG; ++band) 
	    {  
	      FluxOutFull[j].Mag[band] += sign * L90[Comp][band] * DBoost[Comp][band] * FluxOutFull[j].RV[Comp]/299792458.0 /* v/c */;
	    }
	}
    }

  return;
}






